---
sidebar_position: 1
slug: /welcome
---

# Welcome

Welcome to the *Microprocessor Architecture* class.

During the class you will learn how to design and implement a hardware device. 

You takeaways from the course are:
  - learn how computer hardware works;
  - learning how a microprocessor (*MCU*) works, using as an example the RP2040 MCU from the [Raspberry Pi Pico W](https://www.raspberrypi.com/products/raspberry-pi-pico/);
  - building the hardware device either using a breadboard, a prototyping board or a printed circuit board (*PCB*);
  - learning how to develop the software that controls hardware in [Rust](https://www.rust-lang.org/);
  - presenting the hardware device at the *PM Fair* show.


## Team

<table>
<tr valign="top">
<td>
**Alexandru Radovici**
![Alexandru Radovici](images/alexandru_radovici.jpg)

Course Professor \
GitHub: [alexandruradovici](https://github.com/alexandruradovici) \
alexandru.radovici@upb.ro
</td>

<td>
**Gabriel Păvăloiu**
![Gabriel Păvăloiu](images/gabriel_pavaloiu.jpg)
 
Lab Professor \
GitHub: [GabrielPavaloiu](https://github.com/GabrielPavaloiu) \
gabriel.pavaloiu@upb.ro
</td>

<td>
**Teodor Dicu**
![Teodor Dicu](images/teodor_dicu.jpg)
 
Lab Professor\
Responsible for hardware\
GitHub: [DTeodor-Alexaandru](https://github.com/DTeodor-Alexaandru) \
teodor.dicu@wyliodrin.com
</td>
</tr>

<tr valign="top">
<td>
**Dănuț Aldea**
![Danut Aldea](images/danut_aldea.jpg)
 
Lab Professor \
Responsible for software
GitHub: [danutaldea](https://github.com/danutaldea) \
danut.aldea@oxidos.io
</td>

<td>
**Andrei Zamfir**
![Andrei Zamfir](images/andrei_zamfir.jpg)
 
Lab Professor \
Responsible for hardware \
GitHub: [ZamfirAndreiPaul](https://github.com/ZamfirAndreiPaul) \
andrei.zamfir@wyliodrin.com
</td>

<td>
**Ioana Culic**
![Ioana Culic](images/ioana_culic.jpg)

Lab Professor \
GitHub: [ioanaculic](https://github.com/ioanaculic) \
ioana_maria.culic@upb.ro
</td>
</tr>

<tr valign="top">
<td>
**Alexandru Ungureanu**
![Alexandru Ungureanu](images/alexandru_ungureanu.jpg)

Lab Professor \
GitHub: [AlexU01](https://github.com/AlexU01) \
alexagungureanu@gmail.com
</td>

<td>
**Layla El-Ghoul**
![Layla El-Ghoul](images/layla_elghoul.jpg)

Lab Professor \
GitHub: [ScarletWtc](https://github.com/ScarletWtc) \
layla.el@stud.fils.upb.ro
</td>

<td>
**Cristiana Precup**
![Cristiana Precup](images/cristiana_precup.png)

Lab Professor \
GitHub: [cristianaprecup](https://github.com/cristianaprecup) \
cristiana.precup@wyliodrin.com
</td>
</tr>

<tr valign="top">
<td>
**Andrei Batasev**
![Andrei Batasev](images/andrei_batasev.jpg)

Lab Professor \
GitHub: [Pfat8equalsD](https://github.com/Pfat8equalsD) \
andrei.batasev@stud.acs.upb.ro
</td>
</tr>
</table>

## Timetable

| Activity | Group | Day | Hour | Room | Professor |
|----------|-------|-----|------|-------|----------|
| Lecture | *all students* | Friday | 10 - 12 | AN024 | Alexandru Radovici |
| Lab | 1221Aa - first half | Thursday | 8 - 10 | ED 218 | Andrei Zamfir |
| Lab | 1221Ab - second half | Thursday | 12 - 14 | ED 217 | Ioana Culic |
| Lab | 1221Ba - first half | Thursday | 10 - 12 | ED 218 | Cristiana Precup <br/> Layla El-Ghoul |
| Lab | 1221Bb - second half | Thursday | 14 - 16 | ED 217 | Alexandru Ungureanu |
| Lab | 1221Ca - first half | Friday | 14 - 16 | ED 218 | Teodor Dicu |
| Lab | 1221Cb - second half | Thursday | 8 - 10 | ED 217 | Alexandru Ungureanu |
| Lab | 1221D | Thursday | 10 - 12 | ED 217 | Andrei Batasev |
| Lab | 1222EEA | Thursday | 14 - 16 | ED 218 | Dănuț Aldea |
| Lab | 1222EEB | Thursday | 12 - 14 | ED 218 | Teodor Dicu |

## Grading

| Part | Description | Points |
|--------|-------------|--------|
| [Lecture](./category/lecture) tests | You will have a test at every class with subjects from the previous class. | 2p |
| [Lab](./category/lab) | Your work at every lab will be graded. | 2p |
| [Project](./project) | You will have to design and implement a hardware device. Grading will be done for the documentation, hardware design and software development. | 3p |
| Exam | You will have to take an exam during the session. | 4p |
| **Total** | *You will need at least 4.5 points to pass the subject.* | **11p** |

## Rules

1. You can be absent without motivation at a maximum of 2 labs.
2. Labs **CANNOT BE RECOVERED**
3. You can justify your absence from the lab (medical reasons) within a maximum of 2 weeks from the date of your absence
4. Each student must come with their group to the lab, if there are problems you must announce in advance.
5. You can be late for lab up to 10 minutes
6. Students who are catching up on the subject can participate at labs only if there are free places
7. You will need a [github](https://www.github.com) account with your full name and a real profile photo
8. You need at least 4.5 points to pass the exam
 
## Bibliography

### Hardware
1. **Joseph Yiu**, *The Definitive Guide to ARM® Cortex®-M0 and Cortex-M0+ Processors, 2nd Edition* 
2. **Joseph Yiu**, *The Definitive Guide to ARM® Cortex®-M23 and Cortex-M33 Processors* 
3. **Charles Platt**, *Encyclopedia of Electronic Components*, Volumes 1, 2, 3 
4. **Paul Scherz, Simon Monk**, *Practical Electronics for Inventors, 2nd edition*
5. **Raspberry Pi Ltd**, *[RP2040 Datasheet](https://datasheets.raspberrypi.com/rp2040/rp2040-datasheet.pdf)*

### Software
1. **Steve Klabnik, Carol Nichols**, *[The Rust Programming Language](https://doc.rust-lang.org/stable/book/)*
2. **Rust Team**, *[Rust Embedded, The Discovery book](https://docs.rust-embedded.org/discovery/microbit/)*
3. **Raspberry Pi Ltd**, *[Getting started with Raspberry Pi Pico-series](https://datasheets.raspberrypi.com/pico/getting-started-with-pico.pdf)*
4. [embassy_rp](https://docs.embassy.dev/embassy-rp/git/rp2040/index.html) crate documentation
5. [embassy_rp examples](https://github.com/embassy-rs/embassy/tree/main/examples/rp/src/bin)
6. [Raspberry Pi Pico Python SDK](https://datasheets.raspberrypi.com/pico/raspberry-pi-pico-python-sdk.pdf)
7. [Connecting to the Internet with Raspberry Pi Pico W](https://datasheets.raspberrypi.com/picow/connecting-to-the-internet-with-pico-w.pdf)
